@@ -1,4 +1,6 @@
 import os
+import sys
+import platform
 import ctypes
 from ctypes.util import find_library
 
@@ -79,10 +81,8 @@ if os.name == 'nt':
 
     def _load_library(dllname, loadfunction, dllpaths=('', )):
         """Load a DLL via ctypes load function. Return None on failure.
-
         Try loading the DLL from the current package directory first,
         then from the Windows DLL search path.
-
         """
         try:
             dllpaths = (os.path.abspath(os.path.dirname(__file__)),
@@ -107,24 +107,38 @@ if os.name == 'nt':
                     os.environ['PATH'] = oldenv
         return None
 
+
+
+    base_name = 'spatialindex_c'
+    if '64' in platform.architecture()[0]:
+        arch = '64'
+    else:
+        arch = '32'
+
+    lib_name = '%s-%s.dll' % (base_name, arch)
     if 'SPATIALINDEX_C_LIBRARY' in os.environ:
         lib_path, lib_name = os.path.split(os.environ['SPATIALINDEX_C_LIBRARY'])
         rt = _load_library(lib_name, ctypes.cdll.LoadLibrary, (lib_path,))
+    elif 'conda' in sys.version:
+        lib_path = os.path.join(sys.prefix, "Library", "bin")
+        rt = _load_library(lib_name, ctypes.cdll.LoadLibrary, (lib_path,))
     else:
-        rt = _load_library('spatialindex_c.dll', ctypes.cdll.LoadLibrary)
+        rt = _load_library(lib_name, ctypes.cdll.LoadLibrary)
     if not rt:
-        raise OSError("could not find or load spatialindex_c.dll")
+        raise OSError("could not find or load %s" % lib_name)
 
 elif os.name == 'posix':
+
     if 'SPATIALINDEX_C_LIBRARY' in os.environ:
         lib_name = os.environ['SPATIALINDEX_C_LIBRARY']
+        rt = ctypes.CDLL(lib_name)
     else:
         lib_name = find_library('spatialindex_c')
+        rt = ctypes.CDLL(lib_name)
 
-    if lib_name is None:
-        raise OSError("Could not find libspatialindex_c library file")
+    if not rt:
+        raise OSError("Could not load libspatialindex_c library")
 
-    rt = ctypes.CDLL(lib_name)
 else:
     raise RTreeError('Unsupported OS "%s"' % os.name)
 
@@ -206,6 +220,7 @@ rt.Index_Intersects_obj.argtypes = [ctypes.c_void_p,
 rt.Index_Intersects_obj.restype = ctypes.c_int
 rt.Index_Intersects_obj.errcheck = check_return
 
+
 rt.Index_Intersects_id.argtypes = [ctypes.c_void_p,
                                    ctypes.POINTER(ctypes.c_double),
                                    ctypes.POINTER(ctypes.c_double),
@@ -292,6 +307,50 @@ rt.IndexItem_GetID.argtypes = [ctypes.c_void_p]
 rt.IndexItem_GetID.restype = ctypes.c_int64
 rt.IndexItem_GetID.errcheck = check_value
 
+try:
+    rt.Index_GetResultSetOffset.argtypes = [ctypes.c_void_p]
+    rt.Index_GetResultSetOffset.restype = ctypes.c_int64
+    rt.Index_GetResultSetOffset.errcheck = check_value
+
+    rt.Index_SetResultSetOffset.argtypes = [ctypes.c_void_p, ctypes.c_int64]
+    rt.Index_SetResultSetOffset.restype = ctypes.c_int
+    rt.Index_SetResultSetOffset.errcheck = check_return
+
+    rt.Index_GetResultSetLimit.argtypes = [ctypes.c_void_p]
+    rt.Index_GetResultSetLimit.restype = ctypes.c_int64
+    rt.Index_GetResultSetLimit.errcheck = check_value
+
+    rt.Index_SetResultSetLimit.argtypes = [ctypes.c_void_p, ctypes.c_int64]
+    rt.Index_SetResultSetLimit.restype = ctypes.c_int
+    rt.Index_SetResultSetLimit.errcheck = check_return
+
+    rt.Index_Flush.argtypes = [ctypes.c_void_p]
+    rt.Index_Flush.restype = None
+    rt.Index_Flush.errcheck = check_void_done
+
+    rt.Index_Contains_obj.argtypes = [ctypes.c_void_p,
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.c_uint32,
+                                      ctypes.POINTER(
+                                          ctypes.POINTER(ctypes.c_void_p)),
+                                      ctypes.POINTER(ctypes.c_uint64)]
+    rt.Index_Contains_obj.restype = ctypes.c_int
+    rt.Index_Contains_obj.errcheck = check_return
+
+    rt.Index_Contains_id.argtypes = [ctypes.c_void_p,
+                                       ctypes.POINTER(ctypes.c_double),
+                                       ctypes.POINTER(ctypes.c_double),
+                                       ctypes.c_uint32,
+                                       ctypes.POINTER(
+                                           ctypes.POINTER(ctypes.c_int64)),
+                                       ctypes.POINTER(ctypes.c_uint64)]
+    rt.Index_Contains_id.restype = ctypes.c_int
+    rt.Index_Contains_id.errcheck = check_return
+
+except AttributeError:
+    pass
+
 rt.IndexProperty_Create.argtypes = []
 rt.IndexProperty_Create.restype = ctypes.c_void_p
 rt.IndexProperty_Create.errcheck = check_void
@@ -533,3 +592,101 @@ rt.SIDX_DeleteBuffer.restype = None
 rt.SIDX_Version.argtypes = []
 rt.SIDX_Version.restype = ctypes.POINTER(ctypes.c_char)
 rt.SIDX_Version.errcheck = free_returned_char_p
+
+# TPR-Tree API
+try:
+    rt.Index_InsertTPData.argtypes = [ctypes.c_void_p,
+                                      ctypes.c_int64,
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.c_double,
+                                      ctypes.c_double,
+                                      ctypes.c_uint32,
+                                      ctypes.POINTER(ctypes.c_ubyte),
+                                      ctypes.c_uint32]
+    rt.Index_InsertTPData.restype = ctypes.c_int
+    rt.Index_InsertTPData.errcheck = check_return
+
+    rt.Index_DeleteTPData.argtypes = [ctypes.c_void_p,
+                                      ctypes.c_int64,
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.POINTER(ctypes.c_double),
+                                      ctypes.c_double,
+                                      ctypes.c_double,
+                                      ctypes.c_uint32]
+    rt.Index_DeleteTPData.restype = ctypes.c_int
+    rt.Index_DeleteTPData.errcheck = check_return
+
+    rt.Index_TPIntersects_id.argtypes = [ctypes.c_void_p,
+                                         ctypes.POINTER(ctypes.c_double),
+                                         ctypes.POINTER(ctypes.c_double),
+                                         ctypes.POINTER(ctypes.c_double),
+                                         ctypes.POINTER(ctypes.c_double),
+                                         ctypes.c_double,
+                                         ctypes.c_double,
+                                         ctypes.c_uint32,
+                                         ctypes.POINTER(
+                                             ctypes.POINTER(ctypes.c_int64)),
+                                         ctypes.POINTER(ctypes.c_uint64)]
+    rt.Index_TPIntersects_id.restype = ctypes.c_int
+    rt.Index_TPIntersects_id.errcheck = check_return
+
+    rt.Index_TPIntersects_obj.argtypes = [ctypes.c_void_p,
+                                          ctypes.POINTER(ctypes.c_double),
+                                          ctypes.POINTER(ctypes.c_double),
+                                          ctypes.POINTER(ctypes.c_double),
+                                          ctypes.POINTER(ctypes.c_double),
+                                          ctypes.c_double,
+                                          ctypes.c_double,
+                                          ctypes.c_uint32,
+                                          ctypes.POINTER(
+                                              ctypes.POINTER(ctypes.c_void_p)),
+                                          ctypes.POINTER(ctypes.c_uint64)]
+    rt.Index_TPIntersects_obj.restype = ctypes.c_int
+    rt.Index_TPIntersects_obj.errcheck = check_return
+
+    rt.Index_TPIntersects_count.argtypes = [ctypes.c_void_p,
+                                            ctypes.POINTER(ctypes.c_double),
+                                            ctypes.POINTER(ctypes.c_double),
+                                            ctypes.POINTER(ctypes.c_double),
+                                            ctypes.POINTER(ctypes.c_double),
+                                            ctypes.c_double,
+                                            ctypes.c_double,
+                                            ctypes.c_uint32,
+                                            ctypes.POINTER(ctypes.c_uint64)]
+
+    rt.Index_TPNearestNeighbors_id.argtypes = [
+        ctypes.c_void_p,
+        ctypes.POINTER(ctypes.c_double),
+        ctypes.POINTER(ctypes.c_double),
+        ctypes.POINTER(ctypes.c_double),
+        ctypes.POINTER(ctypes.c_double),
+        ctypes.c_double,
+        ctypes.c_double,
+        ctypes.c_uint32,
+        ctypes.POINTER(
+            ctypes.POINTER(ctypes.c_int64)),
+        ctypes.POINTER(ctypes.c_uint64)]
+    rt.Index_TPNearestNeighbors_id.restype = ctypes.c_int
+    rt.Index_TPNearestNeighbors_id.errcheck = check_return
+
+    rt.Index_TPNearestNeighbors_obj.argtypes = [
+        ctypes.c_void_p,
+        ctypes.POINTER(ctypes.c_double),
+        ctypes.POINTER(ctypes.c_double),
+        ctypes.POINTER(ctypes.c_double),
+        ctypes.POINTER(ctypes.c_double),
+        ctypes.c_double,
+        ctypes.c_double,
+        ctypes.c_uint32,
+        ctypes.POINTER(
+            ctypes.POINTER(ctypes.c_void_p)),
+        ctypes.POINTER(ctypes.c_uint64)]
+    rt.Index_TPNearestNeighbors_obj.restype = ctypes.c_int
+    rt.Index_TPNearestNeighbors_obj.errcheck = check_return
+except AttributeError:
+    pass
